#ifndef _BOOLBALL_MODEL_H
#define _BOOLBALL_MODEL_H

#include <glib-object.h>
#include <boardmodel.h>

#define BOOLBALL_MODEL_TYPE		(boolball_model_get_type())
#define BOOLBALL_MODEL(obj)		(G_TYPE_CHECK_INSTANCE_CAST((obj), BOOLBALL_MODEL_TYPE, BoolballModel))

enum {
	CELL_EMPTY = 0,
	CELL_ARROW_LEFT,
	CELL_ARROW_RIGHT,
	CELL_BALL_BLUE,
	CELL_BALL_RED,
	CELL_BONUS,
	CELL_GRAY,
	CELL_PLAYER_BLUE_EVEN,
	CELL_PLAYER_BLUE_ODD,
	CELL_PLAYER_BLUE_DROP,
	CELL_PLAYER_RED_EVEN,
	CELL_PLAYER_RED_ODD,
	CELL_PLAYER_RED_DROP,
	CELL_TELEPORTER
};

enum {
	MODEL_SMALL = 1,
	MODEL_MEDIUM,
	MODEL_LARGE
};

typedef struct _BoolballModel {
	BoardModel	 parent;
} BoolballModel;

typedef struct _BoolballModelClass {
	BoardModelClass	 parentclass;
} BoolballModelClass;

GType		 boolball_model_get_type	(void);
BoolballModel	*boolball_model_new		(void);

BoolballModel	*boolball_model_copy		(BoolballModel *orig);

void		 boolball_model_set_size	(BoolballModel *model, gint modelsize);
const gchar	*boolball_model_get_size_string	(BoolballModel *model);

/*
 * Finds the first ball (from left to right, top to bottom) that is able to
 * fall.  Returns true if such a ball is found and it's coordinates in x and y,
 * false otherwise.
 */
gboolean	 boolball_model_find_free_ball	(BoolballModel *model, gint *x, gint *y);

/*
 * Returns the coordinates of a random teleporter on the field in x and y.
 */
void		 boolball_model_find_teleporter	(BoolballModel *model, gint *x, gint *y);

/*
 * Fills a cell with the player.
 */
void		 boolball_model_cell_set_player	(BoolballModel *model, gint y, gint color);

/*
 * Fill and clear the model.
 */
void		 boolball_model_fill		(BoolballModel *model);
void		 boolball_model_clear		(BoolballModel *model);

#endif

