/* Copyright (c) 2003 Theo van Klaveren, All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer,
 *    without modification, immediately at the beginning of the file.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: game.c,v 1.4 2005/04/06 10:20:48 klaveren Exp $
 */
#include <glib.h>

#include "game.h"

Game *
game_new(BoolballModel *model)
{
	Game *g;
	gint w, h;

	g = g_new0(Game, 1);
	g->model = model;
	g_object_ref(g->model);

	/* Initialize state */
	g->state = movestate_new();
	g->playing = TRUE;
	g->moving = TRUE;
	g->finishing = FALSE;

	/* Create players */
	board_model_get_dimensions(BOARD_MODEL(g->model), &w, &h);
	g->player_red = player_new(w, PC_RED);
	g->player_blue = player_new(w, PC_BLUE);
	g->player = (g_random_int_range(0, 2) == 1) ?
				g->player_red : g->player_blue;

	return g;
}

void
game_destroy(Game *g)
{
	player_destroy(g->player_blue);
	player_destroy(g->player_red);
	g_object_unref(g->model);
	movestate_destroy(g->state);
	g_free(g);
}

void
game_switch_turns(Game *g)
{
	gint w, h, c;

	/*
	 * Switch turns, reset player position to either the left or
	 * the right of the field, depending on player color.
	 */
	board_model_get_dimensions(BOARD_MODEL(g->model), &w, &h);
	
	if (g->player->color == PC_RED) {
		g->player = g->player_blue;
		g->player_pos = 0;
	} else {
		g->player = g->player_red;
		g->player_pos = w - 1;
	}
	
	/* Copy the player's balls to the model */
	for (c = 0; c < w; c++) {
		board_model_cell_set(BOARD_MODEL(g->model), c, 1,
				     g->player->balls[c]);
	}
	
	/* Lastly, paint the player */
	boolball_model_cell_set_player(g->model, g->player_pos, g->player->color);
}

