#include "graphics.h"

/* Vars */
static gint width = 0;
static gint height = 0;

/* Images */
static GdkPixbuf	*block_arrow_left_pb = NULL;
static GdkPixbuf	*block_arrow_right_pb = NULL;
static GdkPixbuf	*block_gray_pb = NULL;
static GdkPixbuf	*block_bonus_pb = NULL;
static GdkPixbuf	*teleporter_pb = NULL;

static GdkPixbuf	*player_blue_even_pb = NULL;
static GdkPixbuf	*player_blue_odd_pb = NULL;
static GdkPixbuf	*player_blue_drop_pb = NULL;
static GdkPixbuf	*player_red_even_pb = NULL;
static GdkPixbuf	*player_red_odd_pb = NULL;
static GdkPixbuf	*player_red_drop_pb = NULL;

static GdkPixbuf	*ball_blue_pb = NULL;
static GdkPixbuf	*ball_red_pb = NULL;

/* Functions */
static GdkPixbuf	*load_svg	(const gchar *name);

#define LOADPB(name, file)	if (name != NULL)\
					    g_object_unref(G_OBJECT(name)); \
				name = load_svg(file);

gboolean
graphics_load_at_size(gint w, gint h) {
	width = w;
	height = h;

	/* Init graphics */
	LOADPB(block_arrow_left_pb, "block-blue-left.svg");
	LOADPB(block_arrow_right_pb, "block-blue-right.svg");
	LOADPB(block_gray_pb, "block-gray.svg");
	LOADPB(block_bonus_pb, "block-bonus.svg");
	LOADPB(ball_blue_pb, "ball-blue.svg");
	LOADPB(ball_red_pb, "ball-red.svg");
	LOADPB(teleporter_pb, "teleporter.svg");
	LOADPB(player_red_even_pb, "player-red-even.svg");
	LOADPB(player_red_odd_pb, "player-red-odd.svg");
	LOADPB(player_red_drop_pb, "player-red-drop.svg");
	LOADPB(player_blue_even_pb, "player-blue-even.svg");
	LOADPB(player_blue_odd_pb, "player-blue-odd.svg");
	LOADPB(player_blue_drop_pb, "player-blue-drop.svg");
	return TRUE;
}

GdkPixbuf *
graphics_get_for_cell(gint content)
{
	GdkPixbuf *pb;

	switch(content) {
		case CELL_ARROW_LEFT: pb = block_arrow_left_pb; break;
		case CELL_ARROW_RIGHT: pb = block_arrow_right_pb; break;
		case CELL_BALL_BLUE: pb = ball_blue_pb; break;
		case CELL_BALL_RED: pb = ball_red_pb; break;
		case CELL_BONUS: pb = block_bonus_pb; break;
		case CELL_GRAY: pb = block_gray_pb; break;
		case CELL_PLAYER_BLUE_EVEN: pb = player_blue_even_pb; break;
		case CELL_PLAYER_BLUE_ODD: pb = player_blue_odd_pb; break;
		case CELL_PLAYER_BLUE_DROP: pb = player_blue_drop_pb; break;
		case CELL_PLAYER_RED_EVEN: pb = player_red_even_pb; break;
		case CELL_PLAYER_RED_ODD: pb = player_red_odd_pb; break;
		case CELL_PLAYER_RED_DROP: pb = player_red_drop_pb; break;
		case CELL_TELEPORTER: pb = teleporter_pb; break;
		case CELL_EMPTY: pb = NULL; break;
		default: g_assert_not_reached();
	}
	return pb;
}

static GdkPixbuf *
load_svg(const gchar *name)
{
	GdkPixbuf *pb;
	GError *err = NULL;
	gchar *fsname;

	fsname = g_strconcat(DATADIR, "/boolball/", name, NULL);
	pb = gdk_pixbuf_new_from_file_at_size(fsname, width, height, &err);
	g_free(fsname);

	if (err) {
		/* FIXME: Give nice error dialog, explaining that the
		 * graphics file could not be found. */
		g_error(err->message);
		return NULL;
	}
	return pb;
}

