/* Copyright (c) 2003 Theo van Klaveren, All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer,
 *    without modification, immediately at the beginning of the file.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: player.c,v 1.5 2005/04/06 10:20:48 klaveren Exp $
 */
#include <glib.h>

#include "player.h"
#include "boolballmodel.h"

Player *
player_new(int width, PlayerColor color)
{
	Player *p;
	gint fill;
	int i;

	p = g_new(Player, 1);
	p->score = 0;
	p->color = color;
	p->balls = g_malloc(width * sizeof(guchar));
	p->finished = FALSE;
	if (color == PC_RED)
		fill = CELL_BALL_RED;
	else
		fill = CELL_BALL_BLUE;
	for (i = 0; i < width; i++) 
		p->balls[i] = fill;

	return p;
}

void
player_destroy(Player *player)
{
	g_free(player->balls);
	g_free(player);
}

gint
player_get_ball_color(Player *player)
{
	if (player->color == PC_RED)
		return CELL_BALL_RED;
	else
		return CELL_BALL_BLUE;
}

gboolean
player_can_move(Player *player, BoolballModel *model)
{
	int ball_x;
	int width, height;
	
	board_model_get_dimensions(BOARD_MODEL(model), &width, &height);

	/* Check if the player can drop any more balls. */
	for (ball_x = 0; ball_x < width; ball_x++) {
		if (player->balls[ball_x] != CELL_EMPTY &&
		    board_model_cell_get(BOARD_MODEL(model), ball_x, 2) == CELL_EMPTY) {
			return TRUE;
		}
	}

	return FALSE;
}

gboolean
player_can_drop_ball(Player *player, BoolballModel *model, gint ball_x)
{
	/* Is there a ball at that spot? */
	if (player->balls[ball_x] == CELL_EMPTY)
		return FALSE;

	/* Is the spot unblocked (no obstacle under it)? */
	if (board_model_cell_get(BOARD_MODEL(model), ball_x, 2) != CELL_EMPTY)
		return FALSE;

	return TRUE;
}

int
player_count_blocked_balls(Player *player, BoolballModel *model)
{
	int count, blocked;
	int width, height;
	
	board_model_get_dimensions(BOARD_MODEL(model), &width, &height);

	blocked = 0;
	for (count = 0; count < width; count++) {
		if (player->balls[count] != CELL_EMPTY) {
			if (!player_can_drop_ball(player, model, count))
					blocked++;
		}
	}

	return blocked;
}
