#ifndef _BOARD_MODEL_H
#define _BOARD_MODEL_H

#define BOARD_MODEL_TYPE	(board_model_get_type())
#define BOARD_MODEL(obj)	(G_TYPE_CHECK_INSTANCE_CAST((obj), \
				 BOARD_MODEL_TYPE, BoardModel))

/*
 * Implements a simple model of a game board.
 */

typedef struct _BoardModel {
	GObject		 parent;
} BoardModel;

typedef struct _BoardModelClass {
	GObjectClass	 parentclass;

	/* Signals */
	void (*board_size_changed)(BoardModel *);
	void (*board_cell_changed)(BoardModel *, GSList *changes);
} BoardModelClass;

typedef struct _BoardModelChange {
	gint x;
	gint y;
} BoardModelChange;

GType	 	 board_model_get_type		(void);
BoardModel	*board_model_new		(void);

/* Set the dimensions of the board. This will clip the existing board if the
 * new dimensions are smaller, or fill the board with empty cells if it is
 * larger. */
void		 board_model_set_dimensions	(BoardModel *model, gint w, gint h);

/* Returns the current dimensions in *w and *h. */
void		 board_model_get_dimensions	(BoardModel *model, gint *w, gint *h);

/* Freeze and thaw the board model. Freezing will prevent changes from being emitted
 * to the view until thawing. */
void		 board_model_freeze		(BoardModel *model);
void		 board_model_thaw		(BoardModel *model);

/* Set and get a single cell's contents. */
void		 board_model_cell_set		(BoardModel *model, gint x, gint y, gint content);
gint		 board_model_cell_get		(BoardModel *model, gint x, gint y);

/* Clone the contents of a model. */
void		 board_model_clone		(BoardModel *orig, BoardModel *copy);

#endif

