#ifndef _BOARD_VIEW_H
#define _BOARD_VIEW_H

#include "boardmodel.h"

#include <gtk/gtkdrawingarea.h>

#define BOARD_VIEW_TYPE			(board_view_get_type())
#define BOARD_VIEW(obj)			(G_TYPE_CHECK_INSTANCE_CAST((obj), BOARD_VIEW_TYPE, BoardView))
#define BOARD_VIEW_CLASS(obj)		(G_TYPE_CHECK_CLASS_CAST((obj), BOARD_VIEW_TYPE, BoardViewClass))
#define BOARD_VIEW_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS ((obj), BOARD_VIEW_TYPE, BoardViewClass))

/*
 * Implements a scalable view of a game board.
 * The board scales to fit in the available space.
 */

typedef struct _BoardView 	BoardView;
typedef struct _BoardViewClass	BoardViewClass;

struct _BoardView {
	GtkDrawingArea		parent;
};

struct _BoardViewClass {
	GtkDrawingAreaClass	parentclass;

	/*
	 * Signals
	 */

	/* The board has been clicked in cell (cell_x, cell_y). */
	void (*board_clicked)(BoardView *, gint cell_x, gint cell_y);

	/* The board has been resized to (new_width x new_height) pixels per cell. */
	void (*board_resized)(BoardView *, gint new_width, gint new_height);

	/*
	 * Abstract methods
	 */

	/* Implement this to draw the given cell. Do not begin_paint and end_paint on the view.
	 * If this function is not implemented, an empty cell is drawn. */
	void (*draw_cell)(BoardView *view, gint x, gint y, gint width, gint height, gint celltype);
};

GType	 	 board_view_get_type		(void);
GtkWidget	*board_view_new			(void);

void		 board_view_set_model		(BoardView *view, BoardModel *model);
BoardModel	*board_view_get_model		(BoardView *view);

void		 board_view_set_square		(BoardView *view, gboolean square);
gboolean	 board_view_get_square		(BoardView *view);

void		 board_view_draw_empty_cell	(BoardView *view, gint x, gint y, gint width, gint height);

#endif

